#       inarray.bash
#       This file is part of zenpakt

#       Copyright 2011 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
#
#       Zenpakt is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

$DOC
=========
 inarray
=========

-------------------------------
 find elements in a given list
-------------------------------

Synopsis
--------
 **inarray** [**-C**] [**-i** *VAR*] [**-m** *D:C* ] *value list*...

Description
-----------

 find *value* in given a *list* (usually an array). If found return success (see `Return Status`_ ).

Options
-------

 -i VAR  store the index where element occurs the first time in VAR.This value is zero indexed.
 -C      do a case-insensitive search.
 -m D:C  list is a pseudo-multi-dimensional array of dimension D and look only in column C of it.
         C is not zero indexed and starts at 1.

Return Status
-------------

 *0 if the element is found
 *1 if the element is not found
$ENDDOC

$$

inarray()
{
################
#LOCAL VARIABLES
################
local OPTIND=1 OPTARG opt || return 1
declare -A I || return 1
I=(
  [ind]=0
  [variable]=
  [tofind]=
  [dimension]=1
  [col]=1
  [i]=0
 )

###########
#OPTIONS
###########
while getopts :Ci:m: opt ; do
  case $opt in
    C)
      trap 'shopt -u nocasematch' RETURN
      shopt -s nocasematch
      ;;
    i)
      I[variable]=$OPTARG
      ;;
    m)
      if [[ $OPTARG = +([0-9]):+([0-9]) ]] ; then
        IFS=: read I[dimension] I[col] <<<"$OPTARG"
        ((${I[col]} <= 0 || ${I[col]} > ${I[dimension]})) && return 1
      else
        return 1
      fi
      ;;
    \?)
      #print -d "$FUNCNAME: unknown option -$OPTARG"
      return 1
      ;;
    :)
      #print -d "$FUNCNAME: -$OPTARG requires an argument"
      return 1
      ;;
  esac
done

#if we at least have VALUE to look for, even if list is an empty list
#we continue by shifting the parameters or else we give shift a value that
#will make it error
if ! shift $(($# > OPTIND ? OPTIND-1 : -1)) 2>/dev/null ; then
  #print -d "$FUNCNAME: no value provided to look for in the list"
  return 1
fi

I[tofind]=$1
shift

###################
#FIND THE ELEMENT
###################

#look for each element according to the dimension
#the the element index for c'th column of r'th row of an n dimensional array is
#   n*r + c
for (( I[ind]=I[i]*I[dimension]+I[col] ; I[ind] <= $# ; I[ind]=(++I[i])*I[dimension]+I[col] )) ; do
  [[ ${@:I[ind]:1} = "${I[tofind]}" ]] && I[i]=-1 && break
done

((I[i] < 0)) || return 1
[[ ${I[variable]} ]] || return 0
printf -v "${I[variable]}" '%s' "$((I[ind]-1))"
}
