#       zprm.bash
#       This file is part of zenpakt
#
#       Copyright 2010 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
#
#       Zenpankt is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

$DOC

SYNOPSIS
--------
zprm [-c] [-b bkupdir] [-s] [-v] [file]

DESCRIPTION

zprm is the rm function for zenpakt.If files are given as argument remove them or
read from stdin for null delimited list of files and remove them.Files are considered
relative to  ROOT environment variable unless they start with a /, if you want the
path relative to something else append before you provide it to zprm.It always saves
directories in an array and removes all files first and then remove the directories
only if empty

OPTIONS
-------
  -c  <not used anymore>ignored
  -b bkupdir  backup each file by calling backup() and using bkupdir  as backup directory
  -s   system remove, remove man and cat pages and dont try to remove anything under install/


$ENDDOC

$$

zprm() {
  #allows non priniting even when VERBOSE is 1, if specifically asked for
  _zprm_print_() { ((verbose)) && print "$@" ;}

  _zprm_ifor_() {
    # ifor successmsg errmsg simplecommand args
    if "${@:3}" 2>/dev/null ; then
      [[ $1 ]] && _zprm_print_ "DELETED:$1"
      return 0
    else
      [[ $2 ]] && _zprm_print_ -e "FAILED:$2"
      return 1
    fi
  }

  _zprm_remove_() {

    if ((system)) ; then
      [[ $1 = install/* ]] && return
      local r="\(.*/\)man\(./[^/]*$\)"
      [[ $1 =~ $r ]] && { verbose=0 _zprm_remove_ "${BASH_REMATCH[1]}cat${BASH_REMATCH[2]}" || return $? ;}
    fi

    if [[ -L $1 ]] ; then
      if ((bkup)) ; then
        _zprm_ifor_ "removed file '$1'" "failed to remove file '$1'" \
          bkup -r "$1" "$bkupdir" "$bkupdir/backup/dobackup-link.list" "$bkupdir/backup/dobackup-all.list"
      else
        _zprm_ifor_ "removed file '$1'" "failed to remove file '$1'" rm -f -- "$1"
      fi
    elif [[ -d $1 ]] ; then
      _zprm_ifor_ "removed empty directory '$1'" "failed to remove '$1' , directory probably not empty" rmdir "$1"

	  elif [[ -f $1 ]] ; then
      if ((bkup)) ; then
         dir=$bkupdir/${1#$ROOT} ; dir=${dir%/*}
         mkdir -p "$dir"
         _zprm_ifor_ "removed file '$1'" "failed to create backup of '$1'..." \
           bkup -r "$1" "$bkupdir" "$bkupdir/backup/dobackup-link.list" "$bkupdir/backup/dobackup-all.list" || return 1

       else
         _zprm_ifor_ "removed file '$1'" "failed to remove file '$1'" rm -f -- "$1"
       fi

    else
       _zprm_print_ -e"SKIPPED: '$1' doesnot exist"
    fi

    return 0
  }

  local bkup dir i save=0 verbose=0 OPTIND=1 REPLY dirs=()|| return 99

  while getopts :cb:sv REPLY ; do
    case $REPLY in
	     #c is ignored now
      c)
	: convert=1
	;;
      b)
	[[ -d $OPTARG ]] || { print -d"$FUNCNAME: backup dir $OPTARG doesnot exist"; return 1;}
	bkup=1 bkupdir=$OPTARG
	;;
      v)
	verbose=1
	;;
      s)
	system=1
	;;
      \?)
	print -d"$FUNCNAME: unknown option -$OPTARG"
	return 99
	;;
      :)
	print -d"$FUNCNAME: option -$OPTARG requires an argument"
	return 99
	;;
    esac
  done

  shift $((OPTIND-1))

  if (($#)) ; then
    for REPLY do
      [[ $REPLY = /* ]] || REPLY=$ROOT$REPLY
      #save directories for later
      if [[ -d $REPLY && ! -L $REPLY ]] ; then
	dirs+=("$REPLY")
	continue
      fi
      #remove the files
      _zprm_remove_ "$REPLY" || return 1
    done

  else
    while read -rd '' ; do
      [[ $REPLY = /* ]] || REPLY=$ROOT$REPLY
      #if system removal is set, directores will be removed if empty after removing file
      if [[ -d $REPLY && ! -L $REPLY ]] ; then
	dirs+=("$REPLY")
	continue
      fi
      _zprm_remove_ "$REPLY" || return 1
    done
  fi

  if ((${#dirs[@]})) ; then
    for i in "${dirs[@]}" ; do
      _zprm_remove_ "$i" || return 1
    done
  fi
  return 0
}
